#include <iostream>
#include <tao/corba.h>
#include <tao/Messaging/Messaging.h>
#include <tao/Strategies/advanced_resource.h>
#include "registryC.h"

class TimeoutApplier
{
public:
   typedef CORBA::ULong Time_in_ms_t;
   TimeoutApplier(CORBA::ORB_ptr orb):orb_(CORBA::ORB::_duplicate(orb)) {}
   CORBA::Object_ptr applytimeout(CORBA::Object_ptr obj,Time_in_ms_t timeout);
private:
  CORBA::ORB_var orb_;
};

CORBA::Object_ptr TimeoutApplier::applytimeout(CORBA::Object_ptr obj, Time_in_ms_t timeout)
{
  TimeBase::TimeT tbttimeout = timeout*10000;
  try
  {
     CORBA::Any relative_CORBA_timeout;
     relative_CORBA_timeout <<= tbttimeout;
     CORBA::PolicyList policy_list(1);
     policy_list.length(1);
     policy_list[0]=orb_->create_policy (Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE, relative_CORBA_timeout); 
     return obj->_set_policy_overrides (policy_list, CORBA::ADD_OVERRIDE);
  }
  catch (CORBA::SystemException& ex)
  {
    return obj;
  }
}

int main(int argc,char **argv)
{
  
  CORBA::ORB_var orb;
  
  try
  {
     orb=CORBA::ORB_init(argc,argv,"tao-orb");
     if(argc < 3)
     {
       std::clog << "Aufruf: client IOR name " << std::endl;
       exit(1);
     }
     TimeoutApplier toa(orb.ptr());
     CORBA::Object_var regobj = orb->string_to_object(argv[1]);
     CORBA::Object_var regobjt = toa.applytimeout(regobj.ptr(),3000); 
     Example::Registry_var reg = Example::Registry::_narrow(regobjt.in());
     CORBA::String_var name=CORBA::string_dup(argv[2]);
     reg->registername(name.in());
  }
  catch (CORBA::Exception &ex)
  {
    std::clog << "CORBA exception raised!" << std::endl;
    ex._tao_print_exception("CORBA Exception:");
  }
  return 0;
}
