#include <iostream>
#include <ace/Thread_Manager.h>
#include <tao/corba.h>
#include <tao/IORTable/IORTable.h>
#include "registryI.h"

void *run_orb(void *arg)
{
    try
    {
       CORBA::ORB_ptr orb = reinterpret_cast<CORBA::ORB_ptr>(arg);
       orb->run();
    }
    catch(CORBA::Exception& ex)
    {
      ex._tao_print_exception("CORBA Exception:");
    }
    return 0;
}

int ACE_TMAIN(int argc, ACE_TCHAR **argv)
{

   try
   {
      CORBA::ORB_var orb = CORBA::ORB_init(argc,argv,"tao");
      /* Aktivieren des Objects */
      CORBA::Object_var poaobj = orb->resolve_initial_references("RootPOA");
      PortableServer::POA_var rootpoa = PortableServer::POA::_narrow(poaobj.in());
      PortableServer::POAManager_var poa_manager =  rootpoa->the_POAManager(); 
      /*
       * Erzeugen eines persistenten POAs
       * bei jeder Inkarnation wollen wir genau die selbe Adresse haben
       */
      CORBA::PolicyList pl;
      pl.length(2);
      pl[0] = rootpoa->create_lifespan_policy(PortableServer::PERSISTENT);
      /*
       * USER_ID, da wir jedesmal die gleiche ID festlegen wollen
       */
      pl[1] = rootpoa->create_id_assignment_policy(PortableServer::USER_ID);
      PortableServer::POA_var ppoa = rootpoa->create_POA("persistent",poa_manager.in(),pl);
      
      /* Instanzierung unsers Objectes */
      PortableServer::Servant serv = new Example_Registry_i();
      CORBA::String_var idstr = (const char *)"reg";
      PortableServer::ObjectId_var reg_oid = PortableServer::string_to_ObjectId(idstr.in());

      ppoa->activate_object_with_id(reg_oid.in(),serv);
      CORBA::Object_var obj = ppoa->id_to_reference(reg_oid.in()); 
      

      /*
       * Nun aktivieren wir beide POAs
       */
      poa_manager->activate();

      /* Ausdrucken der IOR */
      CORBA::String_var iorstr = orb->object_to_string (obj.in());
      std::cout << iorstr << std::endl;

    /* TAO-Spezifisch: IOR-Table */
     CORBA::Object_var iortabobj = orb->resolve_initial_references("IORTable");
     if(CORBA::is_nil(iortabobj.in()))
     {
       std::clog << "Initial reference IORTable not found " << std::endl;
       exit(1);
     }
     
     IORTable::Table_var iortab = IORTable::Table::_narrow(iortabobj.in());
     if (CORBA::is_nil(iortab.in()))
     {
       std::clog << "Can't narrow IORTable " << std::endl;
       exit(1);
     }
     iortab->bind(idstr.in(),iorstr.in());
     std::cout << "corbaloc:iiop:$HOST:$port:/" << idstr.in() << std::endl;

     int pool_size=4;
     ACE_Thread_Manager::instance ()->spawn_n(pool_size,run_orb,(void *)(orb.ptr()),THR_DETACHED | THR_NEW_LWP);
     // Block indefinitely waiting for other threads to exit.
     ACE_Thread_Manager::instance ()->wait ();
     // After run() returns, the ORB has shutdown.
     // Destroy the POA, waiting until the destruction terminates
     rootpoa->destroy (1, 1);
     orb->destroy ();
   }
   catch (CORBA::Exception &ex)
   {
      std::clog << "CORBA exception raised " << std::endl;
      return 1;
   }
   return 0;
}
